from typing import Optional

from .base import *


_default_context: Optional[BaseContext] = None


def set_default_context(context: BaseContext):
    global _default_context
    _default_context = context


def get_default_context() -> BaseContext:
    global _default_context

    if _default_context is None:
        set_default_context(BaseContext())

    return _default_context



import datetime
from typing import TYPE_CHECKING, Optional


if TYPE_CHECKING:
    from pyrtable._baseandtable import _BaseAndTableProtocol


try:
    import pytz
except ImportError:
    pytz = None

from pyrtable.fields import BaseField


class StringField(BaseField):
    def decode_from_airtable(self, value: Optional[str], base_and_table: '_BaseAndTableProtocol') -> str:
        return value or ''

    def encode_to_airtable(self, value: Optional[str]) -> Optional[str]:
        return value or None

    def validate(self, value: Optional[str], base_and_table: '_BaseAndTableProtocol') -> str:
        return value or ''


class IntegerField(BaseField):
    def decode_from_airtable(self, value: Optional[int], base_and_table: '_BaseAndTableProtocol') -> Optional[int]:
        return int(value) if value is not None else None

    def encode_to_airtable(self, value: Optional[int]) -> Optional[int]:
        return value


class FloatField(BaseField):
    def decode_from_airtable(self, value: Optional[float], base_and_table: '_BaseAndTableProtocol') -> Optional[float]:
        return float(value) if value is not None else None

    def encode_to_airtable(self, value: Optional[float]) -> Optional[float]:
        return value


class BooleanField(BaseField):
    def decode_from_airtable(self, value: Optional[bool], base_and_table: '_BaseAndTableProtocol') -> bool:
        return bool(value)

    def encode_to_airtable(self, value: bool) -> Optional[bool]:
        return True if value else None


class DateField(BaseField):
    def decode_from_airtable(self, value: Optional[str], base_and_table: '_BaseAndTableProtocol') \
            -> Optional[datetime.date]:
        if not value:
            return None
        return datetime.datetime.strptime(value, '%Y-%m-%d').date()

    def encode_to_airtable(self, value: Optional[datetime.date]) -> Optional[str]:
        if value is None:
            return None
        return format(value, '%Y-%m-%d')


class DateTimeField(BaseField):
    def decode_from_airtable(self, value, base_and_table: '_BaseAndTableProtocol'):
        if not value:
            return None
        timestamp = datetime.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
        if pytz is not None:
            timestamp = pytz.UTC.localize(timestamp)
        return timestamp

    def encode_to_airtable(self, value):
        if value is None:
            return None

        if pytz is not None:
            value = value.astimezone(pytz.UTC)
        elif value.tzinfo is not None:
            import sys
            print('Warning: Using timezone-aware datetime values require the pytz package', file=sys.stderr)

        return format(value, '%Y-%m-%dT%H:%M:%S.%fZ')


__all__ = ['StringField', 'IntegerField', 'FloatField', 'BooleanField', 'DateField', 'DateTimeField']



import os
import threading
from typing import List, Optional


_config_dirs: Optional[List[str]] = None
_config_dirs_lock = threading.Lock()


def get_config_dirs() -> List[str]:
    global _config_dirs

    import os

    with _config_dirs_lock:
        if _config_dirs is None:
            _config_dirs = os.environ.get('PYRTABLE_CONFIG_DIRS')
            if _config_dirs is not None:
                _config_dirs = _config_dirs.split(':')

        if _config_dirs is None:
            home_dir = os.path.expanduser('~')
            _config_dirs = [
                os.path.join(os.getcwd(), 'config'),
                os.path.join(home_dir, '.config', 'airtable'),
                '/etc/airtable']

    general_config_dir = os.environ.get('CONFIG_DIR')
    if general_config_dir is not None:
        _config_dirs.insert(0, general_config_dir)

    return _config_dirs


def find_config_file(filename: str) -> str:
    for candidate_dir in get_config_dirs():
        if not os.path.isdir(candidate_dir):
            continue
        path = os.path.join(candidate_dir, filename)
        if os.path.isfile(path):
            return path

    raise FileNotFoundError('Cannot find config file %r' % filename)


def _load_config_file(filename: str, loader=None):
    path = find_config_file(filename)
    if not path:
        return None

    if loader is None:
        ext = os.path.splitext(path)[1].lower()
        if ext in ('.yaml', '.yml'):
            # noinspection PyShadowingNames
            def loader(path: str):
                import yaml

                with open(path, 'rt', encoding='utf-8') as fd:
                    return yaml.safe_load(fd)

        elif ext == '.json':
            # noinspection PyShadowingNames
            def loader(path: str):
                import simplejson as json

                with open(path, 'rb') as fd:
                    return json.load(fd, encoding='utf-8')

    if loader is None:
        raise AttributeError('`loader` is None and could not be inferred')

    return loader(path)


_cached_config_files = {}
_cached_config_files_lock = threading.Lock()


def load_config_file(filename: str, loader=None, disable_caching=False):
    global _cached_config_files

    if disable_caching:
        return _load_config_file(filename, loader)

    with _cached_config_files_lock:
        if filename in _cached_config_files:
            return _cached_config_files[filename]

        contents = _load_config_file(filename, loader)
        _cached_config_files[filename] = contents
        return contents


__all__ = ['get_config_dirs', 'find_config_file', 'load_config_file']
